@@ -98,6 +98,7 @@ struct _GDesktopAppInfo
   guint hidden          : 1;
   guint terminal        : 1;
   guint startup_notify  : 1;
+  guint no_fuse         : 1;
   /* FIXME: what about StartupWMClass ? */
 };
 
@@ -256,6 +257,7 @@ g_desktop_app_info_new_from_keyfile (GKeyFile *key_file)
   info->path = g_key_file_get_string (key_file, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_PATH, NULL);
   info->terminal = g_key_file_get_boolean (key_file, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_TERMINAL, NULL) != FALSE;
   info->startup_notify = g_key_file_get_boolean (key_file, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_STARTUP_NOTIFY, NULL) != FALSE;
+  info->no_fuse = g_key_file_get_boolean (key_file, G_KEY_FILE_DESKTOP_GROUP, "X-GIO-NoFuse", NULL) != FALSE;
   info->hidden = g_key_file_get_boolean (key_file, G_KEY_FILE_DESKTOP_GROUP, G_KEY_FILE_DESKTOP_KEY_HIDDEN, NULL) != FALSE;
   
   info->icon = NULL;
@@ -565,6 +567,7 @@ expand_macro (char              macro,
   char *expanded;
   gboolean force_file_uri;
   char force_file_uri_macro;
+  char *uri;
 
   g_return_if_fail (exec != NULL);
 
@@ -573,20 +576,23 @@ expand_macro (char              macro,
    * running or the URI doesn't have a POSIX file path via FUSE
    * we'll just pass the URI.
    */
-  switch (macro)
+  force_file_uri_macro = macro;
+  force_file_uri = FALSE;
+  if (!info->no_fuse)
     {
-    case 'u':
-      force_file_uri_macro = 'f';
-      force_file_uri = TRUE;
-      break;
-    case 'U':
-      force_file_uri_macro = 'F';
-      force_file_uri = TRUE;
-      break;
-    default:
-      force_file_uri_macro = macro;
-      force_file_uri = FALSE;
-      break;
+      switch (macro)
+	{
+	case 'u':
+	  force_file_uri_macro = 'f';
+	  force_file_uri = TRUE;
+	  break;
+	case 'U':
+	  force_file_uri_macro = 'F';
+	  force_file_uri = TRUE;
+	  break;
+	default:
+	  break;
+	}
     }
 
   switch (macro)
@@ -597,15 +603,18 @@ expand_macro (char              macro,
     case 'n':
       if (uris)
 	{
-          if (!force_file_uri)
+	  uri = uris->data;
+          if (!force_file_uri ||
+	      /* Pass URI if it contains an anchor */
+	      strchr (uri, '#') != NULL)
             {
-              expanded = expand_macro_single (macro, uris->data);
+              expanded = expand_macro_single (macro, uri);
             }
           else
             {
-              expanded = expand_macro_single (force_file_uri_macro, uris->data);
+              expanded = expand_macro_single (force_file_uri_macro, uri);
               if (expanded == NULL)
-                expanded = expand_macro_single (macro, uris->data);
+                expanded = expand_macro_single (macro, uri);
             }
 
 	  if (expanded)
@@ -624,15 +633,19 @@ expand_macro (char              macro,
     case 'N':
       while (uris)
 	{
-          if (!force_file_uri)
+	  uri = uris->data;
+	  
+          if (!force_file_uri ||
+	      /* Pass URI if it contains an anchor */
+	      strchr (uri, '#') != NULL)
             {
-              expanded = expand_macro_single (macro, uris->data);
+              expanded = expand_macro_single (macro, uri);
             }
           else
             {
-              expanded = expand_macro_single (force_file_uri_macro, uris->data);
+              expanded = expand_macro_single (force_file_uri_macro, uri);
               if (expanded == NULL)
-                expanded = expand_macro_single (macro, uris->data);
+                expanded = expand_macro_single (macro, uri);
             }
 
 	  if (expanded)
@@ -1300,7 +1313,7 @@ update_mimeapps_list (const char  *desktop_id,
 	        list[i++] = g_strdup (old_list[j]);
 	      else if (add_non_default)
 		{
-		  /* If adding as non-default, and its already in,
+		  /* If adding as non-default, and it's already in,
 		     don't change order of desktop ids */
 		  add_non_default = FALSE;
 		  list[i++] = g_strdup (old_list[j]);
