@@ -20,12 +20,13 @@
  * Author: Alexander Larsson <alexl@redhat.com>
  */
 
-#include <config.h>
+#include "config.h"
 #include <string.h>
 
 #include "gfilemonitor.h"
 #include "gio-marshal.h"
 #include "gioenumtypes.h"
+#include "gfile.h"
 #include "gvfs.h"
 #include "glibintl.h"
 
@@ -34,22 +35,28 @@
 /**
  * SECTION:gfilemonitor
  * @short_description: File Monitor
- * @include: gio.h
+ * @include: gio/gio.h
  *
  * Monitors a file or directory for changes.
  *
  * To obtain a #GFileMonitor for a file or directory, use
- * g_file_monitor_file() or g_file_monitor_directory().
+ * g_file_monitor(), g_file_monitor_file(), or
+ * g_file_monitor_directory().
  *
  * To get informed about changes to the file or directory you
  * are monitoring, connect to the #GFileMonitor::changed signal.
  **/
 
+G_LOCK_DEFINE_STATIC(cancelled);
+
 enum {
   CHANGED,
   LAST_SIGNAL
 };
 
+/* work around a limitation of the aliasing foo */
+#undef g_file_monitor
+
 G_DEFINE_ABSTRACT_TYPE (GFileMonitor, g_file_monitor, G_TYPE_OBJECT);
 
 typedef struct {
@@ -117,7 +124,9 @@ g_file_monitor_get_property (GObject    *object,
       break;
 
     case PROP_CANCELLED:
+      G_LOCK (cancelled);
       g_value_set_boolean (value, priv->cancelled);
+      G_UNLOCK (cancelled);
       break;
 
     default:
@@ -152,9 +161,8 @@ g_file_monitor_finalize (GObject *object)
     }
 
   g_hash_table_destroy (monitor->priv->rate_limiter);
-  
-  if (G_OBJECT_CLASS (g_file_monitor_parent_class)->finalize)
-    (*G_OBJECT_CLASS (g_file_monitor_parent_class)->finalize) (object);
+
+  G_OBJECT_CLASS (g_file_monitor_parent_class)->finalize (object);
 }
 
 static void
@@ -165,11 +173,9 @@ g_file_monitor_dispose (GObject *object)
   monitor = G_FILE_MONITOR (object);
 
   /* Make sure we cancel on last unref */
-  if (!monitor->priv->cancelled)
-    g_file_monitor_cancel (monitor);
-  
-  if (G_OBJECT_CLASS (g_file_monitor_parent_class)->dispose)
-    (*G_OBJECT_CLASS (g_file_monitor_parent_class)->dispose) (object);
+  g_file_monitor_cancel (monitor);
+
+  G_OBJECT_CLASS (g_file_monitor_parent_class)->dispose (object);
 }
 
 static void
@@ -246,9 +252,15 @@ g_file_monitor_init (GFileMonitor *monitor)
 gboolean
 g_file_monitor_is_cancelled (GFileMonitor *monitor)
 {
+  gboolean res;
+
   g_return_val_if_fail (G_IS_FILE_MONITOR (monitor), FALSE);
 
-  return monitor->priv->cancelled;
+  G_LOCK (cancelled);
+  res = monitor->priv->cancelled;
+  G_UNLOCK (cancelled);
+  
+  return res;
 }
 
 /**
@@ -266,10 +278,16 @@ g_file_monitor_cancel (GFileMonitor* monitor)
   
   g_return_val_if_fail (G_IS_FILE_MONITOR (monitor), FALSE);
   
+  G_LOCK (cancelled);
   if (monitor->priv->cancelled)
-    return TRUE;
+    {
+      G_UNLOCK (cancelled);
+      return TRUE;
+    }
   
   monitor->priv->cancelled = TRUE;
+  G_UNLOCK (cancelled);
+  
   g_object_notify (G_OBJECT (monitor), "cancelled");
 
   klass = G_FILE_MONITOR_GET_CLASS (monitor);
@@ -302,6 +320,60 @@ g_file_monitor_set_rate_limit (GFileMonitor *monitor,
     }
 }
 
+typedef struct {
+  GFileMonitor      *monitor;
+  GFile             *child;
+  GFile             *other_file;
+  GFileMonitorEvent  event_type;
+} FileChange;
+
+static gboolean
+emit_cb (gpointer data)
+{
+  FileChange *change = data;
+  g_signal_emit (change->monitor, signals[CHANGED], 0,
+		 change->child, change->other_file, change->event_type);
+  return FALSE;
+}
+
+static void
+file_change_free (FileChange *change)
+{
+  g_object_unref (change->monitor);
+  g_object_unref (change->child);
+  if (change->other_file)
+    g_object_unref (change->other_file);
+  
+  g_slice_free (FileChange, change);
+}
+
+static void
+emit_in_idle (GFileMonitor      *monitor,
+	      GFile             *child,
+	      GFile             *other_file,
+	      GFileMonitorEvent  event_type)
+{
+  GSource *source;
+  FileChange *change;
+
+  change = g_slice_new (FileChange);
+
+  change->monitor = g_object_ref (monitor);
+  change->child = g_object_ref (child);
+  if (other_file)
+    change->other_file = g_object_ref (other_file);
+  else
+    change->other_file = NULL;
+  change->event_type = event_type;
+
+  source = g_idle_source_new ();
+  g_source_set_priority (source, 0);
+
+  g_source_set_callback (source, emit_cb, change, (GDestroyNotify)file_change_free);
+  g_source_attach (source, NULL);
+  g_source_unref (source);
+}
+
 static guint32
 get_time_msecs (void)
 {
@@ -337,9 +409,8 @@ rate_limiter_send_virtual_changes_done_now (GFileMonitor *monitor,
 {
   if (limiter->send_virtual_changes_done_at != 0)
     {
-      g_signal_emit (monitor, signals[CHANGED], 0,
-		     limiter->file, NULL,
-		     G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT);
+      emit_in_idle (monitor, limiter->file, NULL,
+		    G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT);
       limiter->send_virtual_changes_done_at = 0;
     }
 }
@@ -351,9 +422,9 @@ rate_limiter_send_delayed_change_now (GFileMonitor *monitor,
 {
   if (limiter->send_delayed_change_at != 0)
     {
-      g_signal_emit (monitor, signals[CHANGED], 0,
-		     limiter->file, NULL,
-		     G_FILE_MONITOR_EVENT_CHANGED);
+      emit_in_idle (monitor, 
+		    limiter->file, NULL,
+		    G_FILE_MONITOR_EVENT_CHANGED);
       limiter->send_delayed_change_at = 0;
       limiter->last_sent_change_time = time_now;
     }
@@ -522,6 +593,8 @@ update_rate_limiter_timeout (GFileMonitor *monitor,
  * Emits the #GFileMonitor::changed signal if a change
  * has taken place. Should be called from file monitor 
  * implementations only.
+ *
+ * The signal will be emitted from an idle handler.
  **/
 void
 g_file_monitor_emit_event (GFileMonitor      *monitor,
@@ -549,7 +622,7 @@ g_file_monitor_emit_event (GFileMonitor      *monitor,
 	    rate_limiter_send_virtual_changes_done_now (monitor, limiter);
 	  update_rate_limiter_timeout (monitor, 0);
 	}
-      g_signal_emit (monitor, signals[CHANGED], 0, child, other_file, event_type);
+      emit_in_idle (monitor, child, other_file, event_type);
     }
   else
     {
@@ -578,7 +651,7 @@ g_file_monitor_emit_event (GFileMonitor      *monitor,
       
       if (emit_now)
 	{
-	  g_signal_emit (monitor, signals[CHANGED], 0, child, other_file, event_type);
+	  emit_in_idle (monitor, child, other_file, event_type);
 	  
 	  limiter->last_sent_change_time = time_now;
 	  limiter->send_delayed_change_at = 0;
