@@ -244,6 +244,15 @@ canonicalize_filename (const char *filename)
 
   start = (char *)g_path_skip_root (canon);
 
+  if (start == NULL)
+    {
+      /* This shouldn't really happen, as g_get_current_dir() should
+	 return an absolute pathname, but bug 573843 shows this is
+	 not always happening */
+      g_free (canon);
+      return g_build_filename (G_DIR_SEPARATOR_S, filename, NULL);
+    }
+  
   /* POSIX allows double slashes at the start to
    * mean something special (as does windows too).
    * So, "//" != "/", but more than two slashes
@@ -619,7 +628,6 @@ g_local_file_get_child_for_display_name (GFile        *file,
 static const char *
 get_fs_type (long f_type)
 {
-
   /* filesystem ids taken from linux manpage */
   switch (f_type) 
     {
@@ -635,6 +643,8 @@ get_fs_type (long f_type)
       return "befs";
     case 0x1BADFACE:
       return "bfs";
+    case 0x9123683E:
+      return "btrfs";
     case 0xFF534D42:
       return "cifs";
     case 0x73757245:
@@ -693,6 +703,8 @@ get_fs_type (long f_type)
       return "romfs";
     case 0x517B:
       return "smb";
+    case 0x73717368:
+      return "squashfs";
     case 0x012FF7B6:
       return "sysv2";
     case 0x012FF7B5:
@@ -954,14 +966,13 @@ g_local_file_query_filesystem_info (GFile         *file,
 #endif
   block_size = statfs_buffer.f_bsize;
   
-#if defined(__linux__)
-  /* ncpfs does not know the amount of available and free space *
-   * assuming ncpfs is linux specific, if you are on a non-linux platform
-   * where ncpfs is available, please file a bug about it on bugzilla.gnome.org
+  /* Many backends can't report free size (for instance the gvfs fuse
+     backend for backend not supporting this), and set f_bfree to 0,
+     but it can be 0 for real too. We treat the availible == 0 and
+     free == 0 case as "both of these are invalid".
    */
-  if (statfs_buffer.f_bavail == 0 && statfs_buffer.f_bfree == 0 &&
-      /* linux/ncp_fs.h: NCP_SUPER_MAGIC == 0x564c */
-      statfs_buffer.f_type == 0x564c)
+#ifndef G_OS_WIN32
+  if (statfs_buffer.f_bavail == 0 && statfs_buffer.f_bfree == 0)
     no_size = TRUE;
 #endif
   
@@ -1340,7 +1351,7 @@ g_local_file_delete (GFile         *file,
       int errsv = errno;
 
       /* Posix allows EEXIST too, but the more sane error
-	 is G_IO_ERROR_NOT_FOUND, and its what nautilus
+	 is G_IO_ERROR_NOT_FOUND, and it's what nautilus
 	 expects */
       if (errsv == EEXIST)
 	errsv = ENOTEMPTY;
