@@ -5,12 +5,14 @@
 #error "config.h must be included prior to glibintl.h"
 #endif
 
+G_CONST_RETURN gchar *glib_gettext (const gchar *str);
+
 #ifdef ENABLE_NLS
 
-gchar *_glib_gettext (const gchar *str) G_GNUC_FORMAT (1);
-
 #include <libintl.h>
-#define _(String) _glib_gettext(String)
+#define _(String) glib_gettext(String)
+/* Split out this in the code, but keep it in the same domain for now */
+#define P_(String) glib_gettext(String)
 
 #ifdef gettext_noop
 #define N_(String) gettext_noop(String)
@@ -20,11 +22,17 @@ gchar *_glib_gettext (const gchar *str) G_GNUC_FORMAT (1);
 #else /* NLS is disabled */
 #define _(String) (String)
 #define N_(String) (String)
-#define textdomain(String) (String)
+#define P_(String) (String)
+#define textdomain(String) ((String) ? (String) : "messages")
 #define gettext(String) (String)
 #define dgettext(Domain,String) (String)
 #define dcgettext(Domain,String,Type) (String)
+#define dngettext(Domain,String1,String2,N) ((N) == 1 ? (String1) : (String2))
 #define bindtextdomain(Domain,Directory) (Domain) 
+#define bind_textdomain_codeset(Domain,Codeset)
 #endif
 
+/* not really I18N-related, but also a string marker macro */
+#define I_(string) g_intern_static_string (string)
+
 #endif /* __GLIBINTL_H__ */
